const defaultOptions = {
  color: 'primary',
  isRange: false,
  allowSameDayRange: true,
  lang: 'en', // internationalization
  startDate: undefined,
  endDate: undefined,
  minDate: null,
  maxDate: null,
  disabledDates: [],
  disabledWeekDays: undefined,
  highlightedDates: [],
  weekStart: 0,
  dateFormat: 'MM/DD/YYYY',
  enableMonthSwitch: true,
  enableYearSwitch: true,
  displayYearsCount: 50
};

export default defaultOptions;